@@ -1,7 +1,7 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/calctool.h,v 1.85 2006/10/30 18:12:45 richb Exp $
+/*  $Header$
  *
- *  Copyright (c) 1987-2006 Sun Microsystems, Inc. All Rights Reserved.
+ *  Copyright (c) 1987-2007 Sun Microsystems, Inc. All Rights Reserved.
  *           
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
-#define MP_SIZE      150     /* Size of the multiple precision values. */
+#define MP_SIZE      1000     /* Size of the multiple precision values. */
 
 #define FCLOSE       (void) fclose     /* To make lint happy. */
 #define FPRINTF      (void) fprintf
@@ -54,7 +54,8 @@
 /* Menu bar menu types. */
 
 enum mb_type { M_ABOUT, M_ASCII, M_BASIC, M_ADV, M_CONTENTS, M_COPY,  M_FIN,
-	       M_PASTE, M_QUIT,  M_REGS,  M_SCI, M_EXP, M_TSEP, M_ZEROES };
+	       M_PASTE, M_QUIT,  M_REGS,  M_SCI, M_EXP, M_TSEP, M_ZEROES,
+               M_LR_ARITH, M_OP_ARITH };
 
 enum base_type { BIN, OCT, DEC, HEX };      /* Base definitions. */
 
@@ -66,8 +67,7 @@ enum item_type { BASEITEM, TTYPEITEM, NUMITEM,
 
 /* Popup menu types. */
 enum menu_type { M_ACC,  M_CON,  M_EXCH, M_FUN,  M_LSHF,
-                 M_RCL,  M_RSHF, M_STO,  M_NONE
-};
+                 M_RCL,  M_RSHF, M_STO, M_NONE };
 
 /* Calculator modes. */
 enum mode_type { BASIC, ADVANCED, FINANCIAL, SCIENTIFIC };
@@ -86,81 +86,83 @@ enum trig_type { DEG, GRAD, RAD };          /* Trigonometric types. */
 
 enum trig_func {SIN=0, COS=1, TAN=2};       
 
-/* Abbreviations for the gcalctool keyboard and menu equivalents. */
+/* Abbreviations for the gcalctool keyboard */
 
-#define KEY_DIV   a_buttons[3]              /* / */
-#define KEY_LPAR  a_buttons[4]              /* ( */
-#define KEY_BSP   a_buttons[5]              /* Control-h */
-#define KEY_CE    a_buttons[6]              /* Control-Backspace */
-
-#define KEY_MUL   a_buttons[11]             /* * */
-#define KEY_RPAR  a_buttons[12]             /* ) */
-
-#define KEY_CHS   a_buttons[13]             /* C */
-#define KEY_INT   a_buttons[14]             /* i */
-#define KEY_STO   a_buttons[15]             /* S */
-
-#define KEY_SUB   a_buttons[19]             /* - */
-#define KEY_PER   a_buttons[20]             /* % */
-#define KEY_SQRT  a_buttons[21]             /* s */
-#define KEY_FRAC  a_buttons[22]             /* : */
-#define KEY_RCL   a_buttons[23]             /* R */
-
-#define KEY_EQ    a_buttons[26]             /* = */
-#define KEY_ADD   a_buttons[27]             /* + */
-#define KEY_REC   a_buttons[28]             /* r */
-#define KEY_SQR   a_buttons[29]             /* @ */
-#define KEY_ABS   a_buttons[30]             /* u */
-#define KEY_EXCH  a_buttons[31]             /* X */
-
-#define KEY_CTRM  f_buttons[0]              /* m */
-#define KEY_DDB   f_buttons[1]              /* D */
-#define KEY_FV    f_buttons[2]              /* v */
-#define KEY_PMT   f_buttons[3]              /* P */
-#define KEY_PV    f_buttons[4]              /* p */
-#define KEY_RATE  f_buttons[5]              /* T */
-#define KEY_SLN   f_buttons[6]              /* l */
-#define KEY_SYD   f_buttons[7]              /* Y */
-#define KEY_TERM  f_buttons[8]              /* T */
-
-#define KEY_LSFT  s_buttons[0]              /* < */
-#define KEY_RSFT  s_buttons[1]              /* > */
-#define KEY_16    s_buttons[2]              /* [ */
-#define KEY_32    s_buttons[3]              /* ] */
-#define KEY_MOD   s_buttons[6]              /* M */
-#define KEY_ACC   s_buttons[7]              /* A */
-
-#define KEY_CON   s_buttons[8]              /* # */
-#define KEY_FUN   s_buttons[9]              /* F */
-#define KEY_ETOX  s_buttons[11]             /* { */
-#define KEY_TTOX  s_buttons[12]             /* } */
-#define KEY_YTOX  s_buttons[13]             /* y */
-#define KEY_FACT  s_buttons[14]             /* ! */
-#define KEY_RAND  s_buttons[15]             /* ? */
-
-#define KEY_COS   s_buttons[19]             /* J */
-#define KEY_SIN   s_buttons[20]             /* K */
-#define KEY_TAN   s_buttons[21]             /* L */
-#define KEY_LN    s_buttons[22]             /* N */
-#define KEY_LOG   s_buttons[23]             /* G */
-
-#define KEY_OR    s_buttons[27]             /* | */
-#define KEY_AND   s_buttons[28]             /* & */
-#define KEY_NOT   s_buttons[29]             /* ~ */
-#define KEY_XOR   s_buttons[30]             /* ^ */
-#define KEY_XNOR  s_buttons[31]             /* n */
-
-#define BCOLS          4      /* No of columns of Basic Mode buttons. */
-#define BROWS          5      /* No of rows of Basic Mode buttons. */
-
-#define ACOLS          8      /* No of columns of Advanced Mode buttons. */
-#define AROWS          4      /* No of rows of Advanced Mode buttons. */
-
-#define FCOLS          8      /* No of columns of Financial Mode buttons. */
-#define FROWS          2      /* No of rows of Financial Mode buttons. */
-
-#define SCOLS          8      /* No of columns of Scientific Mode buttons. */
-#define SROWS          4      /* No of rows of Scientific Mode buttons. */
+enum
+{
+    KEY_0,
+    KEY_1,
+    KEY_2,
+    KEY_3,
+    KEY_4,
+    KEY_5,
+    KEY_6,
+    KEY_7,
+    KEY_8,
+    KEY_9,
+    KEY_A,
+    KEY_B,
+    KEY_C,
+    KEY_D,
+    KEY_E,
+    KEY_F,
+    KEY_NUMERIC_POINT,
+    KEY_CALCULATE,
+    KEY_CLEAR,
+    KEY_CLEAR_ENTRY,
+    KEY_START_BLOCK,
+    KEY_END_BLOCK,
+    KEY_ADD,
+    KEY_SUBTRACT,
+    KEY_MULTIPLY,
+    KEY_DIVIDE,
+    KEY_BACKSPACE,
+    KEY_CHANGE_SIGN,
+    KEY_INTEGER,
+    KEY_FRACTION,
+    KEY_PERCENTAGE,
+    KEY_SQUARE,
+    KEY_SQUARE_ROOT,
+    KEY_RECIPROCAL,
+    KEY_E_POW_X,
+    KEY_10_POW_X,
+    KEY_X_POW_Y,
+    KEY_FACTORIAL,
+    KEY_RANDOM,
+    KEY_SINE,
+    KEY_COSINE,
+    KEY_TANGENT,
+    KEY_NATURAL_LOGARITHM,
+    KEY_LOGARITHM,
+    KEY_ABSOLUTE_VALUE,
+    KEY_MASK_16,
+    KEY_MASK_32,
+    KEY_MODULUS_DIVIDE,
+    KEY_EXPONENTIAL,
+    KEY_NOT,
+    KEY_OR,
+    KEY_AND,
+    KEY_XOR,
+    KEY_XNOR,
+    KEY_FINC_CTRM,
+    KEY_FINC_DDB,
+    KEY_FINC_FV,
+    KEY_FINC_PMT,
+    KEY_FINC_PV,
+    KEY_FINC_RATE,
+    KEY_FINC_SLN,
+    KEY_FINC_SYD,
+    KEY_FINC_TERM,
+    KEY_LEFT_SHIFT,
+    KEY_RIGHT_SHIFT,
+    KEY_STORE,
+    KEY_RECALL,
+    KEY_EXCHANGE,
+    KEY_ACCURACY_MENU,
+    KEY_CONSTANTS_MENU,
+    KEY_FUNCTIONS_MENU,
+    NKEYS
+};
 
 #define EQUAL(a, b)    (strlen(a)==strlen(b)) & !strcmp(a, b) 
 
@@ -175,7 +177,7 @@ enum trig_func {SIN=0, COS=1, TAN=2};
 #endif /*lint*/
 #endif /*LINT_CAST*/
 
-#define MAX_DIGITS     40         /* Maximum displayable number of digits. */
+#define MAX_DIGITS     200         /* Maximum displayable number of digits. */
 #define MAX_LOCALIZED  (MAX_DIGITS * (1 + MB_LEN_MAX) + MB_LEN_MAX)
 
 /* Maximum number of various graphics pieces. */
@@ -186,7 +188,7 @@ enum trig_func {SIN=0, COS=1, TAN=2};
 #define MAXLINE        512        /* Length of character strings. */
 #endif /*MAXLINE*/
 
-#define MAXACC         30    /* Max. number of digits after numeric point. */
+#define MAXACC         99     /* Max. number of digits after numeric point. */
 #define MAXBASES       4          /* Maximum number of numeric bases. */
 #define MAXCONFUN      10         /* Maximum number of constants/functions. */
 #define MAXDISPMODES   3          /* Maximum number of display modes. */
@@ -207,11 +209,6 @@ enum trig_func {SIN=0, COS=1, TAN=2};
 #define MIN(x,y)       ((x) < (y) ? (x) : (y))
 #endif /*MIN*/
 
-#define B_NOBUTTONS      BROWS * BCOLS  /* Number of Basic Mode buttons. */
-#define A_NOBUTTONS      AROWS * ACOLS  /* Number of Advanced Mode buttons. */
-#define F_NOBUTTONS      FROWS * FCOLS  /* Number of Financial Mode buttons. */
-#define S_NOBUTTONS      SROWS * SCOLS  /* Number of Scientific Mode buttons. */
-
 #ifndef CFNAME
 #define CFNAME         ".gcalctoolcf"
 #endif /*CFNAME*/
@@ -253,7 +250,11 @@ enum button_flags {
     expnum       = (1 << 13),  /* Exponential number */
     postfixop    = (1 << 14),  /* Unary postfix operation */
     prefixop     = (1 << 15),  /* Unary prefix operation */
-    dpoint       = (1 << 16)   /* Decimal point */
+    dpoint       = (1 << 16),  /* Decimal point */
+    pending      = (1 << 17),  /* Is a pending button */
+    hex          = (1 << 18),  /* Is a hex button A..F */
+    oct_set      = (1 << 19),  /* Is a oct button 0..7 */
+    bin_set      = (1 << 20)   /* Is a bin button 0..1 */
 };
 
 enum shiftd {
@@ -267,15 +268,11 @@ enum syntax {
 };
 
 struct button {
-    char *str;               /* Button display string. */
-    char *hstr;              /* Button help string. */
-    char *astr;              /* AccessibleName string (if tooltip not useful) */
+    char *symname;           /* Expression function name */
     guint mods[MAXEXTRAS];   /* Keyboard modifiers (Shift, Ctrl, ...). */
     guint value[MAXEXTRAS];  /* Button keyboard equivalents. */
     char func_char;          /* Unique function string character. */
-    enum menu_type mtype;    /* Type of popup menu (if any). */
     void (*func)();          /* Function to obey on button press. */
-    char *symname;           /* Expression function name */
     enum button_flags flags; /* Misc flags */
 };
 
@@ -353,6 +350,8 @@ struct calcVars {                      /* Calctool variables and options. */
     int doing_mi;      /* Set if adjusting the "show zeroes" menu item. */
     int down;          /* Indicates is a mouse button is down. */
     int error;         /* Indicates some kind of display error. */
+    int ghost_zero;    /* Flag to indicate display with "0", actually 
+                           having empty content. */
     int math_error;    /* Math error (used in expression mode) */
     int hyperbolic;    /* If set, trig functions will be hyperbolic. */
     int iconic;        /* Set if window is currently iconic. */
@@ -543,4 +542,10 @@ void mpstr(int *, int *);
 void mpsub(int *, int *, int *);
 void mptanh(int *, int *);
 
+void make_exp(char *number, int t[MP_SIZE]);
+void exp_replace(char *text);
+void insert_to_cursor(char *text);
+void get_expr_from_display();
+void delete_from_cursor();
+
 #endif /*CALCTOOL_H*/
