@@ -1,7 +1,7 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/extern.h,v 1.12 2004/11/11 19:22:41 richb Exp $
+/*  $Header$
  *
- *  Copyright (c) 1987-2005 Sun Microsystems, Inc. All Rights Reserved.
+ *  Copyright (c) 1987-2007 Sun Microsystems, Inc. All Rights Reserved.
  *           
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -42,6 +42,7 @@ extern char *Rdstr[];          /* Display mode X resource strings. */
 extern char *Rmstr[];          /* Mode mode X resource strings. */
 extern char *Rtstr[];          /* Trig mode X resource strings. */
 extern char *Rsstr[];          /* Syntax resource strings. */
+extern char *Rcstr[];          /* Bitcalculating mode. */
 
 extern double max_fix[];       /* Maximum showable fixed values. */
 
@@ -49,9 +50,6 @@ extern int basevals[];         /* Supported arithmetic bases. */
 extern int validkeys[];        /* Valid keys after an error condition. */
 extern int cur_pos[];          /* Current positions - left/right handed. */
 
-extern struct button b_buttons[];         /* Basic mode button values. */
-extern struct button a_buttons[];         /* Advanced mode button values. */
-extern struct button f_buttons[];         /* Financial mode button values. */
-extern struct button s_buttons[];         /* Scientific mode button values. */
+extern struct button buttons[];         /* Calculator button values. */
 
 #endif /*EXTERN_H*/
